@@ -7,12 +7,13 @@ use std::{
 
 use dunce::simplified;
 use path_slash::PathExt;
+use yaml_rust::{Yaml, YamlEmitter, YamlLoader};
 
 use crate::{
     artifacts_emitter::ArtifactsEmitter,
     error::BuildError,
     plugins::Plugins,
-    util::{get_artifacts_dir, run_command},
+    util::{copy_to, get_artifacts_dir, run_command},
     BuildResult, FileOperation, IOResultExt,
 };
 
@@ -106,14 +107,15 @@ impl Flutter {
                 .join("package_config_subset"),
         )?;
 
-        Self::copy(
-            self.root_dir.join("pubspec.yaml"),
-            flutter_out_root.join("pubspec.yaml"),
+        let assets = self.copy_pubspec_yaml(
+            &self.root_dir.join("pubspec.yaml"),
+            &flutter_out_root.join("pubspec.yaml"),
         )?;
 
+        self.precache()?;
         self.run_flutter_assemble(&flutter_out_root)?;
         self.emit_flutter_artifacts(&flutter_out_root)?;
-        self.emit_flutter_checks(&local_roots).unwrap();
+        self.emit_flutter_checks(&local_roots, &assets).unwrap();
 
         if Self::build_mode() == "profile" {
             cargo_emit::rustc_cfg!("flutter_profile");
@@ -122,6 +124,61 @@ impl Flutter {
         Ok(())
     }
 
+    fn link_asset(
+        &self,
+        from_dir: &Path,
+        to_dir: &Path,
+        asset: &str,
+    ) -> BuildResult<Option<PathBuf>> {
+        let mut segments = asset.split("/");
+        if let Some(first) = segments.next() {
+            if first != "packages" {
+                let asset = from_dir.join(first);
+                copy_to(&asset, to_dir, true)?;
+                return Ok(Some(asset));
+            }
+        }
+        Ok(None)
+    }
+
+    // copy pub_spec.yaml, linking assets in the process; asset directories
+    // need to be linked relative to pubspec.yaml
+    // Returns asset directories
+    fn copy_pubspec_yaml(&self, from: &Path, to: &Path) -> BuildResult<Vec<PathBuf>> {
+        let mut res = Vec::<PathBuf>::new();
+
+        let pub_spec = fs::read_to_string(from).wrap_error(FileOperation::Read, || from.into())?;
+        let pub_spec = YamlLoader::load_from_str(&pub_spec)
+            .map_err(|err| BuildError::YamlError { source: err })?;
+
+        let from_dir = from.parent().unwrap();
+        let to_dir = to.parent().unwrap();
+
+        let flutter = &pub_spec[0];
+        if let Yaml::Hash(hash) = flutter {
+            let flutter = hash.get(&Yaml::String("flutter".into()));
+            if let Some(Yaml::Hash(flutter)) = flutter {
+                let assets = flutter.get(&Yaml::String("assets".into()));
+                if let Some(Yaml::Array(assets)) = assets {
+                    for asset in assets {
+                        if let Yaml::String(str) = asset {
+                            if let Some(asset) = self.link_asset(from_dir, to_dir, str)? {
+                                res.push(asset);
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
+        let mut out_str = String::new();
+        let mut emitter = YamlEmitter::new(&mut out_str);
+        emitter.dump(flutter).unwrap();
+
+        Self::copy(from, to)?;
+        Ok(res)
+    }
+
     pub fn build_mode() -> String {
         let mut build_mode: String = std::env::var("PROFILE").unwrap();
         let profile = std::env::var("FLUTTER_PROFILE").unwrap_or_else(|_| "false".into());
@@ -341,7 +398,7 @@ impl Flutter {
         Ok(())
     }
 
-    fn emit_flutter_checks(&self, roots: &HashSet<PathBuf>) -> BuildResult<()> {
+    fn emit_flutter_checks(&self, roots: &HashSet<PathBuf>, assets: &[PathBuf]) -> BuildResult<()> {
         cargo_emit::rerun_if_changed! {
             self.root_dir.join("pubspec.yaml").to_str().unwrap(),
             self.root_dir.join("pubspec.lock").to_str().unwrap(),
@@ -351,6 +408,16 @@ impl Flutter {
             self.emit_checks_for_dir(path)?;
         }
 
+        for asset in assets {
+            if asset.is_dir() {
+                self.emit_checks_for_dir(asset)?;
+            } else {
+                cargo_emit::rerun_if_changed! {
+                    asset.to_string_lossy()
+                }
+            }
+        }
+
         cargo_emit::rerun_if_env_changed!("FLUTTER_PROFILE");
 
         Ok(())
@@ -380,6 +447,47 @@ impl Flutter {
             || from.as_ref().into(),
         )
     }
+
+    pub fn precache(&self) -> BuildResult<()> {
+        // no precaching necessary for local engines
+        if self.options.local_engine.is_some() {
+            return Ok(());
+        }
+        let engine_version = ArtifactsEmitter::find_flutter_bin()
+            .ok_or_else(|| BuildError::OtherError("Couldn't find Flutter installation".into()))?
+            .join("internal")
+            .join("engine.version");
+
+        let engine_version = fs::read_to_string(&engine_version)
+            .wrap_error(FileOperation::Read, || engine_version)?;
+
+        let last_engine_version_path = self.out_dir.join("last_precached_engine_version");
+        if last_engine_version_path.exists() {
+            let last_engine_version = fs::read_to_string(&last_engine_version_path)
+                .wrap_error(FileOperation::Read, || last_engine_version_path.clone())?;
+            if last_engine_version == engine_version {
+                return Ok(());
+            }
+        }
+
+        // need to run flutter precache
+        let mut command = self.create_flutter_command();
+        command
+            .arg("precache")
+            .arg("-v")
+            .arg("--suppress-analytics")
+            .arg(match self.target_os {
+                TargetOS::Mac => "--macos",
+                TargetOS::Windows => "--windows",
+                TargetOS::Linux => "--linux",
+            });
+        self.run_flutter_command(command)?;
+
+        fs::write(&last_engine_version_path, &engine_version)
+            .wrap_error(FileOperation::Write, || last_engine_version_path)?;
+
+        Ok(())
+    }
 }
 
 #[derive(serde::Serialize, serde::Deserialize, Debug)]
