@@ -35,6 +35,16 @@ pub(super) fn symlink<P: AsRef<Path>, Q: AsRef<Path>>(src: P, dst: Q) -> BuildRe
         } else {
             std::os::windows::fs::symlink_file(&src, &dst)
         };
+        if let Err(error) = &res {
+            if error.raw_os_error() == Some(1314) {
+                return Err(BuildError::OtherError(
+                    "Unable to create a symlink. Please enable developer mode:\n\
+                    https://docs.microsoft.com/en-us/windows/apps/get-started/\
+                    enable-your-device-for-development"
+                        .into(),
+                ));
+            }
+        }
         res.wrap_error_with_src(
             FileOperation::SymLink,
             || dst.as_ref().into(),
@@ -91,10 +101,11 @@ where
         if allow_symlinks {
             let src_can = canonicalize(src.as_ref())
                 .wrap_error(FileOperation::Canonicalize, || src.as_ref().into())?;
-            let dst_can = canonicalize(src.as_ref())
-                .wrap_error(FileOperation::Canonicalize, || src.as_ref().into())?;
+            let dst_can = canonicalize(dst.as_ref())
+                .wrap_error(FileOperation::Canonicalize, || dst.as_ref().into())?;
             if src_can == dst_can {
-                // nothing to do here
+                // nothing to do here; This is useful on windows where often the symlink
+                // can't be deleted and recreated because something is using it
                 return Ok(());
             }
         }
